require 'puppetx'

module Puppetx::Puppetlabs::PEConsole
  class Config

    # Regular expressions for parsing INI format
    SECTION_HEADER    = /^\[(\w+)\s*\]$/
    VALUE_ASSIGNMENT  = /^\s*(\w+)\s*=\s*(\S+)\s*$/
    COMMENT_LINE      = /^\s*[#;]/
    BLANK_LINE        = /^\s*$/

    # Default values for the dashabord terminus config
    def self.defaults
      {
        :server           => 'localhost',
        :certificate_name => 'pe-internal-dashboard',
        :port             => 443,
        :report_path      => '/reports/upload'
      }
    end

    def self.load(config_file = nil)
      config_file ||= File.join(Puppet[:confdir], "console.conf")

      result = parse_ini(config_file)

      legal_keys = defaults.keys
      main_section = result['main'] || {}
      # merge with defaults but filter out anything except the legal settings
      config_hash = defaults.merge(main_section).reject do |k, v|
        !(legal_keys.include?(k))
      end

      config_hash[:server] = config_hash[:server].strip
      config_hash[:certificate_name] = config_hash[:certificate_name].strip
      config_hash[:report_path] = config_hash[:report_path].strip
      config_hash[:port] = config_hash[:port].to_i

      self.new(config_hash)
    rescue => detail
      puts detail.backtrace if Puppet[:trace]
      Puppet.warning "Could not configure classifier terminus: #{detail}"
      raise
    end

    def self.parse_ini(config_file)
      if File.exists?(config_file)
        Puppet.debug("Configuring classifier terminus with config file #{config_file}")
        content = File.read(config_file)
      else
        Puppet.debug("No #{config_file} file found; falling back to default server and port #{defaults[:server]}:#{defaults[:port]}")
        content = ''
      end

      result = {}
      section = nil
      content.lines.each_with_index do |line,number|
        # Gotta track the line numbers properly
        number += 1
        case line
          when SECTION_HEADER
            section = $1
            result[section] ||= {}
          when VALUE_ASSIGNMENT
            raise "Setting '#{line}' is illegal outside of section in classifier config #{config_file}:#{number}" unless section
            result[section][$1.to_sym] = $2
          when COMMENT_LINE
            # Skip comments
          when BLANK_LINE
            # Skip blank lines
          else
            raise "Unparseable line '#{line}' in classifier config #{config_file}:#{number}"
        end
      end
      result
    end

    def initialize(config_hash = {})
      @config = config_hash
    end

    # Configuration Accessors

    def server
      config[:server]
    end

    def certificate_name
      config[:certificate_name]
    end

    def port
      config[:port]
    end

    def report_path
      config[:report_path]
    end

    # Private instance methods
    private

    attr_reader :config

  end
end
